﻿using System;
using System.Security;
using System.Web.UI;
using Itslearning.Platform.RestApi.Sdk.Common;
using Itslearning.Platform.RestApi.Sdk.Common.Entities;
using Itslearning.Platform.RestApi.Sdk.LearningToolApp;
using Itslearning.Platform.RestApi.Sdk.LearningToolApp.Entities;

namespace LearningToolTemplate
{
    /// <summary>
    /// Base class for all pages to ease communicating with itslearning API.
    /// </summary>
    public abstract class BasePage : Page
    {
        protected void CreateSession()
        {
            // You can initialize communication with itslearning as shown in SampleCommunicationHelper.InitApiSession.
            // 
            // What the method does:
            // 1. Checks if the request is not expired. If it is expired, SecurityAccessDeniedException is thrown.
            // 2. Validates the signature. If it is not valid, SecurityAccessDeniedException is thrown.
            // 3. Puts ApiSessionId into your application session state.
            // 4. Puts ApiSession object into your application session state (it it useful for RestApi.Client .NET wrapper assembly).
            // 5. Puts all other received parameters into your application session state 
            //    (UserId, FirstName, LastName, Locale, Language, Use12hTimeFormat, 
            //    Accessibility, Permissions, 
            //    WindowsTimeZoneId, OlsonTimeZoneId).
            //
            // To validate timestamp and signature the method reads application key, 
            // shared secret and request lifetime from a sourceimplementing IApplicationSettings interface. 
            // DefaultSettings class reads the settings from web.config (appSettings section).
            SampleCommunicationHelper.InitApiSession(this, "WRITE YOUR APPLICATION KEY HERE", "WRITE YOUR SHARED SECRET HERE", 15);


            // Or let code provided by itslearning to do all the stuff above -
            // use CommunicationHelper.InitApiSession.
            //
            // To validate timestamp and signature the method reads application key, 
            // shared secret and request lifetime from a source implementing IApplicationSettings interface. 
            // DefaultSettings class reads the settings from web.config (appSettings section).
            //CommunicationHelper.InitApiSession(this, new DefaultSettings());

			
            // If the application is licensed, you can get license information for current user here
            // TODO: uncomment the following line if you need license info
            //AppLicenses appLicenses = SampleLicenseHelper.GetAppLicensesForCurrentUser(ApiSession);
			
        }

    	
        /// <summary>
        /// True, if API session has been created for the current user and put in to session.
        /// </summary>
        protected bool IsApiSessionCreated
        {
			get
			{

				int? sessionUserId = null;

				if (UserInfo != null)
				{
					sessionUserId = UserInfo.UserId;
				}
				try
				{
					int newuserId = int.Parse(Request["UserId"]);

					if (sessionUserId != null && newuserId == sessionUserId && ApiSessionId != null)
					{
						return true;
					}
				}
				catch
				{
					//Erroneous input. Clear session.
				}
				Session.Clear();

				return false;
			}
        }

        /// <summary>
        /// API session ID.
        /// </summary>
        protected string ApiSessionId
        {
            get
            {
                // You can get it directly from the session state
                return Session[LearningObjectId + ";" + LearningObjectInstanceId + ";ApiSessionId"] as string;

                // Or you can use the helper method
                //return CommunicationHelper.GetApiSessionId(this);
            }
        }

        /// <summary>
        /// API session object. It is used by RestApi.Client proxy.
        /// </summary>
        protected ApiSession ApiSession
        {
            get
            {
                // You can get it directly from the session state
                return Session[LearningObjectId + ";" + LearningObjectInstanceId + ";ApiSession"] as ApiSession;

                // Or you can use the helper method
                //return CommunicationHelper.GetApiSession(this);
            }
        }

        /// <summary>
        /// Permissions the user has on the learning object instance.
        /// </summary>
        protected LearningObjectInstancePermissions Permissions
        {
            get
            {
                // You can get it directly from the session state
                var permissionsAsObject = Session[LearningObjectId + ";" + LearningObjectInstanceId + ";Permissions"];
                if (permissionsAsObject == null)
                {
                    return LearningObjectInstancePermissions.None;
                }
                return (LearningObjectInstancePermissions) permissionsAsObject;

                // Or you can use the helper method
                //return CommunicationHelper.GetPermissions(this);
            }
        }

        /// <summary>
        /// Will return true if user should be allowed to read content, should always be true as long as
        /// authentication has been successful
        /// </summary>
        protected bool HasReadPermission
        {
            get
            {
                return (Permissions & LearningObjectInstancePermissions.Read) == LearningObjectInstancePermissions.Read;
            }
        }

        /// <summary>
        /// Only used for "Learning activity" applications. "Learning resource" applications will always return false here.
        /// Will return true if a user has permission to participate, it should be checked to e.g. decide if user
        /// should be allowed to submit answer to a test. Students and teachers in an itslearning course will normally 
        /// have this permission.
        /// </summary>
        protected bool HasParticipatePermission
        {
            get
            {
                return (Permissions & LearningObjectInstancePermissions.Participate) == LearningObjectInstancePermissions.Participate;
            }
        }

        /// <summary>
        /// Only used for "Learning activity" applications. "Learning resource" applications will always return false here.
        /// Will return true if a user has permission to evaluate, it should be checked to e.g. decide if user
        /// should be allowed to review and evaluate answers to a test. Teachers in an itslearning course will normally 
        /// have this permission.
        /// </summary>
        protected bool HasEvaluatePermission
        {
            get
            {
                return (Permissions & LearningObjectInstancePermissions.Evaluate) == LearningObjectInstancePermissions.Evaluate;
            }
        }

        /// <summary>
        /// Will return true if user should be allowed to edit instance content stored in your application and also e.g. update deadline
        /// stored in itslearning (use rest api method). Teachers in an itslearning course will normally have this permission.
        /// If your application supports sharing - the user will not have Modify permission when accessing an instance that
        /// is connected to a library "master" instance, even though user is author. Shared content must be edited from the library.
        /// </summary>
        protected bool HasModifyPermission
        {
            get
            {
                return (Permissions & LearningObjectInstancePermissions.Modify) == LearningObjectInstancePermissions.Modify;
            }
        }

        /// <summary>
        /// Will only be used if your application supports sharing.
        /// Will return true if user is allowed to e.g. update deadline stored in itslearning (use rest api method). But
        /// user should _not_ be allowed to edit instance content stored in your application with this permission.
        /// HasModifyInstancePermission can only be true if user is accessing a shared instance inside a course.
        /// </summary>
        protected bool HasModifyInstancePermission
        {
            get
            {
                return (Permissions & LearningObjectInstancePermissions.ModifyInstance) == LearningObjectInstancePermissions.ModifyInstance;
            }
        }

        /// <summary>
        /// Information about user (id, first name, last name, culture, language, etc).
        /// </summary>
        protected UserInfo UserInfo
        {
            get
            {
                // You can get it directly from the session state
                return Session["UserInfo"] as UserInfo;

                // Or you can use the helper method
                //return CommunicationHelper.GetUserInfo(this);
            }
        }

        /// <summary>
        /// Id of current learning object passed in the query string.
        /// It is expected and recommended to pass LearningObjectId between all pages in your application.
        /// </summary>
        /// <remarks>
        /// This Id can be the same for several learning object instances. Use this Id to store application content 
        /// that should be shared between instances.
        /// </remarks>
        protected int LearningObjectId
        {
            get
            {
                // You can get LearningObjectId directly from the query string
                string learningObjectIdAsString = Request.QueryString["LearningObjectId"];
                return int.Parse(learningObjectIdAsString);

                // Or you can use the helper method
                //return CommunicationHelper.GetLearningObjectId(this);
            }
        }

        /// <summary>
        /// Id of the current learning object instance passed in the query string. 
        /// It is expected and recommended to pass LearningObjectInstanceId between all pages in your application.
        /// </summary>
        /// <remarks>This Id is unique. It should be passed to most of the API methods.</remarks>
        protected int LearningObjectInstanceId
        {
            get
            {
                // You can get LearningObjectInstanceId directly from the query string
                string learningObjectInstanceIdAsString = Request.QueryString["LearningObjectInstanceId"];
                return int.Parse(learningObjectInstanceIdAsString);

                // Or you can use the helper method
                //return CommunicationHelper.GetLearningObjectInstanceId(this);
            }
        }


        /// <summary>
        /// THIS IS RESERVED FOR FUTURE USE
        /// Version of the current learning object passed in the query string.
        /// It is expected and recommended to pass Version between all pages in your application.
        /// </summary>
        /// <remarks>
        /// Use this parameter if application supports versioning. It contains 
        /// number of version that is requested or "LatestOrDraft" string to retrive latest version.
        /// </remarks>
        protected string Version
        {
            get
            {
                // You can get Version directly from the query string
                return Request.QueryString["Version"];

                // Or you can use the helper method
                //return CommunicationHelper.GetVersion(this);
            }
        }

        #region sample methods

        // The following methods should give you an idea about what you can do, please modify them as you see fit, move them to
        // your business components and have fun.
        // Since every call to the itslearning rest api is quite costly it's a good idea to make as few calls as possible.


        /// <summary>
        /// Get the possible assessment scales that can be used for current instance. 
        /// This method should only be called if your application is a "Learning activity" with advanced assessment.
        /// </summary>
        /// <returns></returns>
        protected Assessments GetPossibleAssessments()
        {
            using (LearningObjectServiceClient client = new LearningObjectServiceClient(ApiSession))
            {
                return client.GetPossibleAssessments(LearningObjectId, LearningObjectInstanceId);
            }    
        }

        /// <summary>
        /// Returns true if user is allowed to update instance
        /// </summary>
        protected bool CanUpdateInstance
        {
            get
            {
                return HasModifyPermission || HasModifyInstancePermission;
            }
        }
        
        /// <summary>
        /// Update data kept in itslearning for an instance.
        /// The effect of this call is that deadline will be updated in itslearning for current instance and reflected in task lists.
        /// </summary>
        /// <param name="deadlineUtc">deadline in utc - only applicable for application of type "Learning activity"</param>
        /// <param name="activeFromUtc"></param>
        /// <param name="activeToUtc"></param>
        /// <param name="assessmentId">Id of assessment scale to use - only applicable for application of type "Learning activity" with advanced assessment</param>
        protected void UpdateInstance(DateTime? deadlineUtc, DateTime? activeFromUtc, DateTime? activeToUtc, int? assessmentId)
        {
            if (!CanUpdateInstance)
                throw new SecurityException("User is not allowed to update instance");

            using (LearningObjectServiceClient client = new LearningObjectServiceClient(ApiSession))
            {
                LearningObjectInstance instance = client.GetLearningObjectInstance(LearningObjectId, LearningObjectInstanceId);
                instance.DeadlineUtc = deadlineUtc;
                instance.ActiveFromUtc = activeFromUtc;
                instance.ActiveToUtc = activeToUtc;
                instance.AssessmentId = assessmentId;
                client.UpdateLearningObjectInstance(instance, LearningObjectId, LearningObjectInstanceId);
            }
        }

        /// <summary>
        /// Get possible assessment items for an instance that can be used to set assessment for an answer.
        /// This method should only be called if your application is a "Learning activity" with advanced assessment.
        /// IMPORTANT: Before you call this method you should make sure that an AssessmentId has been set for the instance.
        /// </summary>
        /// <returns></returns>
        protected AssessmentItems GetPossibleAssessmentItems()
        {
            using (LearningObjectServiceClient client = new LearningObjectServiceClient(ApiSession))
            {
                return client.GetAssessmentItems(LearningObjectId, LearningObjectInstanceId);
            }
        }

        /// <summary>
        /// Call this to see if user can set assessment. Returns true if user is allowed.
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        protected bool CanSetAssessmentForUser(int userId)
        {
            // This may seem a bit strange... A user can set his/her own grade.
            // But it needs to be possible if e.g. the application is a test that can do automatic evaluation.
            return userId == UserInfo.UserId || HasEvaluatePermission;
        }

        /// <summary>
        /// Update assessment for a user.
        /// This method should only be called if your application is a "Learning activity" with advanced assessment.
        /// The effect of this call is that assessment will be set for a user in itslearning for current instance.
        /// </summary>
        /// <param name="userId">userId for a user who has access to this instance</param>
        /// <param name="assessmentItemId"></param>
        /// <param name="comment"></param>
        protected void SetAssessment(int userId, int assessmentItemId, string comment)
        {
            if (!CanSetAssessmentForUser(userId))
                throw new SecurityException("User is not allowed to set assessment");

            using (LearningObjectServiceClient client = new LearningObjectServiceClient(ApiSession))
            {
                LearningObjectInstanceUserReport report = client.GetLearningObjectInstanceUserReport(LearningObjectId, LearningObjectInstanceId, userId);
                // if you try to call this method for a teacher, you would receive a null object for the report, you may want to handle this
                // by getting a report in a separate method and check
                if (report != null)
                {
                    report.AssessmentItemId = assessmentItemId;
                    report.Comment = comment;
                    client.UpdateLearningObjectInstanceUserReport(report, LearningObjectId, LearningObjectInstanceId, userId);
                }
            }
        }

        #endregion
    }
}