﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.Security.Cryptography;
using System.ServiceModel.Security;
using System.Text;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using Itslearning.Platform.RestApi.Sdk.Common.Entities;
// the following 3 using statements are needed if schools should be retrieved
using Itslearning.Platform.RestApi.Sdk.LearningToolApp;                                             
using Itslearning.Platform.RestApi.Sdk.LearningToolApp.Entities;
using OrganizationType=Itslearning.Platform.RestApi.Sdk.LearningToolApp.Entities.OrganizationType;

namespace LearningToolTemplate
{
    /// <summary>
    /// NOTE: Only provided as a reference. 
    /// NOTE: Not for production usage.
    /// Sample of a Helper class for communication with itslearning API.
    /// </summary>
    internal static class SampleCommunicationHelper
    {
        /// <summary>
        /// Initializes communication between your application and itslearning.
        /// Should be called immediately after receiving request from itslearning.
        /// </summary>
        /// <param name="page">Page to be served.</param>
        /// <param name="applicationKey">Key identifying your application in itslearning.</param>
        /// <param name="sharedSecret">Secret shared between your application and itslearning.</param>
        /// <param name="requestLifetimeInMinutes">For how many minutes the request is valid.</param>
        internal static void InitApiSession(Page page, string applicationKey, string sharedSecret, int requestLifetimeInMinutes)
        {
            var parameters = page.Request.QueryString;

            // Validate if the request is not expired and if the signature is valid
            ValidateQueryString(page.Request.Url.AbsoluteUri, sharedSecret, requestLifetimeInMinutes, parameters);

            // Store received parameters to the session state
            StoreParametersToSession(parameters, page.Session, applicationKey, sharedSecret);
        }

        /// <summary>
        /// Checks if the request signature is valid and the request is not expired.
        /// </summary>
        /// <param name="queryString"></param>
        /// <param name="sharedSecret"></param>
        /// <param name="requestLifetimeInMinutes"></param>
        /// <param name="parameters"></param>
        internal static void ValidateQueryString(string queryString, string sharedSecret, int requestLifetimeInMinutes, NameValueCollection parameters)
        {
            if (string.IsNullOrEmpty(queryString)) throw new ArgumentNullException("queryString");
            if (string.IsNullOrEmpty(sharedSecret)) throw new ArgumentNullException("sharedSecret");
            if (parameters == null) throw new ArgumentNullException("parameters");

            // Check if timestamp is correct and within acceptable range
            string timestampString = parameters["Timestamp"];
            if (string.IsNullOrEmpty(timestampString))
            {
                throw new SecurityAccessDeniedException("Timestamp is not specified.");
            }

            DateTime timestamp;
            if (
                !DateTime.TryParse(timestampString, CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal,
                                   out timestamp))
            {
                throw new SecurityAccessDeniedException(
                    "Query string has an invalid timestamp(check that UTC time is passed and that server time is correct)");
            }

            timestamp = timestamp.ToUniversalTime();

            if (timestamp < DateTime.UtcNow.AddMinutes(-requestLifetimeInMinutes) ||
                timestamp > DateTime.UtcNow.AddMinutes(requestLifetimeInMinutes))
            {
                throw new SecurityAccessDeniedException(
                    "Query string has an invalid timestamp(check that UTC time is passed and that server time is correct)");
            }

            // Check signature
            string signature = parameters["Signature"];
            if (string.IsNullOrEmpty(signature))
            {
                throw new SecurityAccessDeniedException("Signature is not specified.");
            }

            // Remove signature from the querystring
            queryString = HttpUtility.UrlDecode(queryString);
            string queryStringWithoutSignature =
                queryString.Replace(string.Format("&Signature={0}", signature), string.Empty);

            if (ComputeHash(queryStringWithoutSignature + sharedSecret) != signature)
            {
                throw new SecurityAccessDeniedException("Signature is invalid.");
            }
        }

        /// <summary>
        /// Stores parameters received from itslearning into session state.
        /// See TODO for UserInfo.Schools
        /// </summary>
        /// <param name="parameters"></param>
        /// <param name="session"></param>
        /// <param name="applicationKey"></param>
        /// <param name="sharedSecret"></param>
        private static void StoreParametersToSession(NameValueCollection parameters, HttpSessionState session, string applicationKey, string sharedSecret)
        {
            // Get data from parameters, validate types
            string learningObjectIdAsString = parameters["LearningObjectId"];
            if (learningObjectIdAsString == null)
                throw new Exception("LearningObjectId is not specified in the querystring.");
            int learningObjectId = int.Parse(learningObjectIdAsString);

            string learningObjectInstanceIdAsString = parameters["LearningObjectInstanceId"];
            if (learningObjectInstanceIdAsString == null)
                throw new Exception("LearningObjectInstanceId is not specified in the querystring.");
            int learningObjectInstanceId = int.Parse(learningObjectInstanceIdAsString);

            string apiSessionId = parameters["ApiSessionId"];
            if (apiSessionId == null) throw new Exception("ApiSessionId is not specified in the querystring.");

            string permissionsAsString = parameters["Permissions"];
            if (permissionsAsString == null) throw new Exception("Permissions is not specified in the querystring.");
            var permissions =
                (LearningObjectInstancePermissions)
                Enum.Parse(typeof (LearningObjectInstancePermissions), permissionsAsString, true);

            string version = parameters["Version"];
            if (version == null) throw new Exception("Version is not specified in the querystring.");

            // User info
            string userIdAsString = parameters["UserId"];
            if (userIdAsString == null) throw new Exception("UserId is not specified in the querystring.");
            int userId = int.Parse(userIdAsString);

            string firstName = parameters["FirstName"];
            if (firstName == null) throw new Exception("FirstName is not specified in the querystring.");

            string lastName = parameters["LastName"];
            if (lastName == null) throw new Exception("LastName is not specified in the querystring.");

            string locale = parameters["Locale"];
            if (locale == null) throw new Exception("Locale is not specified in the querystring.");

            string language = parameters["Language"];
            if (language == null) throw new Exception("Language is not specified in the querystring.");

            string use12HTimeFormatAsString = parameters["Use12HTimeFormat"];
            if (use12HTimeFormatAsString == null)
                throw new Exception("Use12hTimeFormat is not specified in the querystring.");
            bool use12HTimeFormat = bool.Parse(use12HTimeFormatAsString);

            string windowsTimeZoneId = parameters["WindowsTimeZoneId"];
            if (windowsTimeZoneId == null)
                throw new Exception("WindowsTimeZoneId is not specified in the querystring.");

            string olsonTimeZoneId = parameters["OlsonTimeZoneId"];
            if (olsonTimeZoneId == null) throw new Exception("OlsonTimeZoneId is not specified in the querystring.");

            string accessibilityAsString = parameters["Accessibility"];
            if (accessibilityAsString == null)
                throw new Exception("Accessibility is not specified in the querystring.");
            bool accessibility = bool.Parse(accessibilityAsString);

            string customerIdAsString = parameters["CustomerId"];
            if (customerIdAsString == null)
                throw new Exception("CustomerId is not specified in the querystring.");
            int customerId = int.Parse(customerIdAsString);

            string roleAsString = parameters["Role"];
			UserRole role = UserRole.Guest;
			if (roleAsString != null)
			{
				role = (UserRole)Enum.Parse(typeof(UserRole), roleAsString, true);
			}

            ApiSession apiSession = ConstructApiSession(apiSessionId, applicationKey, sharedSecret);

            List<SchoolInfo> schools = null;
            // TODO: If you need school info for current user, you should uncomment the following block
            // Note: School info was removed from querystring because it in some cases caused the 
            //       query string to be too long and SSO to application would fail
/*
            using (LearningObjectServiceClient client = new LearningObjectServiceClient(apiSession))
            {
                schools = new List<SchoolInfo>();
                Organizations organizations = client.GetOrganisationsForCurrentUser();
                foreach (Organization organization in organizations)
                {
                    if (organization.Type == OrganizationType.School)
                    {
                        schools.Add(new SchoolInfo{LegalId = organization.LegalId, SchoolId = organization.HierarchyId});
                    }
                }
            }
*/
            
            // The following parameters represent a protected data.
            // To be able to receive these parameters application must be permitted on a site level.
            // If application is not permitted to receive protected data, then these parameters are not transferred.
            string email = parameters["email"];
            string syncKey = parameters["synckey"];
            string mobile = parameters["mobile"];
            string custom1Id = parameters["custom1id"];
            string custom2Id = parameters["custom2id"];
            string custom3Id = parameters["custom3id"];
            string custom4Id = parameters["custom4id"];
            string custom5Id = parameters["custom5id"];
            string custom1 = parameters["custom1"];
            string custom2 = parameters["custom2"];
            string custom3 = parameters["custom3"];
            string custom4 = parameters["custom4"];
            string custom5 = parameters["custom5"];

            var userInfo = new UserInfo
            {
                Accessibility = accessibility,
                FirstName = firstName,
                Language = language,
                LastName = lastName,
                Locale = locale,
                WindowsTimeZoneId = windowsTimeZoneId,
                OlsonTimeZoneId = olsonTimeZoneId,
                UserId = userId,
                Use12HTimeFormat = use12HTimeFormat,
                CustomerId = customerId,
                Schools = schools,    
                Role = role,

                Email = email,
                SyncKey = syncKey,
                Mobile = mobile,
                Custom1Id = custom1Id,
                Custom2Id = custom2Id,
                Custom3Id = custom3Id,
                Custom4Id = custom4Id,
                Custom5Id = custom5Id,
                Custom1 = custom1,
                Custom2 = custom2,
                Custom3 = custom3,
                Custom4 = custom4,
                Custom5 = custom5,
            };

            // Put data into session
            session[learningObjectId + ";" + learningObjectInstanceId + ";ApiSessionId"] = apiSessionId;
            session[learningObjectId + ";" + learningObjectInstanceId + ";ApiSession"] = apiSession;
            session[learningObjectId + ";" + learningObjectInstanceId + ";Permissions"] = permissions;
            session["UserInfo"] = userInfo;
        }

        /// <summary>
        /// Creates ApiSession object to be used by cliet proxies.
        /// </summary>
        /// <param name="apiSessionId"></param>
        /// <param name="applicationKey"></param>
        /// <param name="sharedSecret"></param>
        /// <returns></returns>
        private static ApiSession ConstructApiSession(string apiSessionId, string applicationKey, string sharedSecret)
        {
            ApiSession apiSession = ApiSession.CreateApiSession(apiSessionId);
            apiSession.ApplicationKey = applicationKey;
            apiSession.Hash = ComputeHash(apiSession.ApplicationKey + sharedSecret + apiSession.SessionId + apiSession.TimeStamp);

            return apiSession;
        }

        /// <summary>
        /// Calculates MD5 of the string.
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        internal static string ComputeHash(string input)
        {
            var md5Hasher = new MD5CryptoServiceProvider();
            byte[] data = md5Hasher.ComputeHash(Encoding.Default.GetBytes(input));
            var sb = new StringBuilder();

            // Loop through each byte of the hashed data and format each one as a hexadecimal string.
            for (int i = 0; i < data.Length; i++)
            {
                sb.Append(data[i].ToString("x2"));
            }

            return sb.ToString();
        }
    }
}